@@ -26,7 +26,7 @@ from __future__ import print_function, division, absolute_import
 
 
 __project__ = "pigit"
-__version__ = "1.0.9"
+__version__ = "1.3.0"
 __url__ = "https://github.com/zlj-zz/pigit.git"
 __uri__ = __url__
 
@@ -39,16 +39,16 @@ __copyright__ = "Copyright (c) 2021 Zachary"
 
 import os
 import sys
-import signal
 import argparse
 import logging
 import logging.handlers
 import textwrap
 from distutils.util import strtobool
+from shutil import get_terminal_size
+from typing import Optional
 
 from .log import LogHandle
-from .compat import get_terminal_size
-from .utils import confirm, color_print, leave
+from .utils import confirm, color_print, is_color
 from .common import Color, Fx, TermColor
 from .git_utils import Git_Version, Repository_Path, repository_info, git_local_config
 from .decorator import time_it
@@ -66,7 +66,8 @@ Log = logging.getLogger(__name__)
 #####################################################################
 
 # For windows.
-IS_WIN = sys.platform.lower().startswith("win")
+IS_WIN: bool = sys.platform.lower().startswith("win")
+Log.debug("Runtime platform is windows: {0}".format(IS_WIN))
 if IS_WIN:
     USER_HOME = os.environ["USERPROFILE"]
     PIGIT_HOME = os.path.join(USER_HOME, __project__)
@@ -88,10 +89,20 @@ class ConfigError(Exception):
 
 
 class Config(object):
-    Conf_Path = PIGIT_HOME + "/pigit.conf"
-    Config_Template = textwrap.dedent(
-        """
+    """PIGIT configuration class."""
+
+    _conf_path: str = PIGIT_HOME + "/pigit.conf"  # default config path.
+
+    CONFIG_TEMPLATE: str = textwrap.dedent(
+        """\
         #? Config file for pigit v. {version}
+
+        #  ____ ___ ____ ___ _____                            __ _                       _   _
+        # |  _ \_ _/ ___|_ _|_   _|           ___ ___  _ __  / _(_) __ _ _   _ _ __ __ _| |_(_) ___  _ __
+        # | |_) | | |  _ | |  | |_____ _____ / __/ _ \| '_ \| |_| |/ _` | | | | '__/ _` | __| |/ _ \| '_ \\
+        # |  __/| | |_| || |  | |_____|_____| (_| (_) | | | |  _| | (_| | |_| | | | (_| | |_| | (_) | | | |
+        # |_|  |___\____|___| |_|            \___\___/|_| |_|_| |_|\__, |\__,_|_|  \__,_|\__|_|\___/|_| |_|
+        #                                                          |___/
         # Git-tools -- pigit configuration.
 
         # Show original git command.
@@ -113,6 +124,9 @@ class Config(object):
         # Wether show files that cannot be counted.
         codecounter_show_invalid={codecounter_show_invalid}
 
+        # Wether show files icons. Font support required, like: 'Nerd Font'
+        codecounter_show_icon={codecounter_show_icon}
+
         # Output format of statistical results.
         # Supported: [table, simple]
         # When the command line width is not enough, the `simple ` format is forced.
@@ -135,61 +149,68 @@ class Config(object):
 
         # The max line width when use `-h` get help message.
         help_max_line_width={help_max_line_width}
+
+        # Wether run PIGIT in debug mode.
+        debug_mode={debug_mode}
+
         """
     )
 
     # yapf: disable
-    keys = [
+    _keys :list[str]= [
         'gitprocessor_show_original', 'gitprocessor_use_recommend',
         'gitprocessor_interactive_color', 'gitprocessor_interactive_help_showtime',
-        'codecounter_use_gitignore', 'codecounter_show_invalid', 'codecounter_result_format',
+        'codecounter_use_gitignore', 'codecounter_show_invalid',
+        'codecounter_result_format', 'codecounter_show_icon',
         'gitignore_generator_timeout',
         'repository_show_path', 'repository_show_remote', 'repository_show_branchs',
         'repository_show_lastest_log', 'repository_show_summary',
-        'help_use_color', 'help_max_line_width'
+        'help_use_color', 'help_max_line_width',
+        'debug_mode'
     ]
     # yapf: enable
 
-    gitprocessor_show_original = True
-    gitprocessor_use_recommend = True
-    gitprocessor_interactive_color = True
-    gitprocessor_interactive_help_showtime = 1.5
+    # config default values.
+    gitprocessor_show_original: bool = True
+    gitprocessor_use_recommend: bool = True
+    gitprocessor_interactive_color: bool = True
+    gitprocessor_interactive_help_showtime: float = 1.5
 
-    codecounter_use_gitignore = True
-    codecounter_show_invalid = False
-    codecounter_result_format = "table"  # table, simple
-    _supported_result_format = ["table", "simple"]
+    codecounter_use_gitignore: bool = True
+    codecounter_show_invalid: bool = False
+    codecounter_show_icon: bool = False
+    codecounter_result_format: str = "table"  # table, simple
+    _supported_result_format: list = ["table", "simple"]
 
-    gitignore_generator_timeout = 60
+    gitignore_generator_timeout: int = 60
 
-    repository_show_path = True
-    repository_show_remote = True
-    repository_show_branchs = True
-    repository_show_lastest_log = True
-    repository_show_summary = False
+    repository_show_path: bool = True
+    repository_show_remote: bool = True
+    repository_show_branchs: bool = True
+    repository_show_lastest_log: bool = True
+    repository_show_summary: bool = False
 
-    help_use_color = True
-    help_max_line_width = 90
+    help_use_color: bool = True
+    help_max_line_width: int = 90
+
+    debug_mode: str = False
 
     # Store warning messages.
-    warnings = []
+    warnings: list = []
 
-    def __init__(self, path=None):
+    def __init__(self, path: Optional[str] = None) -> None:
         super(Config, self).__init__()
         if not path:
-            self.config_path = self.Conf_Path
+            self.config_path = self._conf_path
         else:
             self.config_path = path
         conf = self.load_config()
-        # from pprint import pprint
 
-        # print(len(conf))
-        # pprint(conf)
-        for key in self.keys:
+        for key in self._keys:
             if key in conf.keys() and conf[key] != "==error==":
                 setattr(self, key, conf[key])
 
-    def load_config(self):
+    def load_config(self) -> dict:
         new_config = {}
         config_file = self.config_path
         try:
@@ -206,18 +227,21 @@ class Config(object):
                         # invalid line.
                         continue
                     key, line = line.split("=", maxsplit=1)
+                    # processing.
                     key = key.strip()
                     line = line.strip().strip('"')
-                    if key not in self.keys:
+                    # checking.
+                    if key not in self._keys:
+                        self.warnings.append("'{0}' is not be supported!".format(key))
                         continue
-                    if type(getattr(self, key)) == int:
+                    elif type(getattr(self, key)) == int:
                         try:
                             new_config[key] = int(line)
                         except ValueError:
                             self.warnings.append(
                                 'Config key "{0}" should be an integer!'.format(key)
                             )
-                    if type(getattr(self, key)) == bool:
+                    elif type(getattr(self, key)) == bool:
                         try:
                             # True values are y, yes, t, true, on and 1;
                             # false values are n, no, f, false, off and 0.
@@ -229,10 +253,10 @@ class Config(object):
                                     key
                                 )
                             )
-                    if type(getattr(self, key)) == str:
-                        if "color" in key and not self.is_color(line):
+                    elif type(getattr(self, key)) == str:
+                        if "color" in key and not is_color(line):
                             self.warnings.append(
-                                'Config key "{0}" should be RGB, like: #FF0000'.format(
+                                'Config key "{0}" should be RGB string, like: #FF0000'.format(
                                     key
                                 )
                             )
@@ -241,7 +265,7 @@ class Config(object):
         except Exception as e:
             Log.error(str(e))
 
-        if (
+        if (  # check codecounter output format wether supported.
             "codecounter_result_format" in new_config
             and new_config["codecounter_result_format"]
             not in self._supported_result_format
@@ -253,28 +277,39 @@ class Config(object):
                 )
             )
 
+        if "version" in new_config and new_config["version"] != __version__:
+            self.warnings.append(
+                "The current configuration file is not up-to-date."
+                "You'd better recreate it."
+            )
+
         return new_config
 
-    def is_color(self, v):
-        return v and v.startswith("#") and len(v) == 7
-
-    @classmethod
-    def create_config_template(cls):
+    def create_config_template(self) -> None:
         if not os.path.isdir(PIGIT_HOME):
             os.makedirs(PIGIT_HOME, exist_ok=True)
 
-        if os.path.exists(cls.Conf_Path) and not confirm(
+        if os.path.exists(self.config_path) and not confirm(
             "Configuration exists, overwrite? [y/n]"
         ):
             return
+
+        # Try to load already has config.
+        conf = self.load_config()
+        for key in self._keys:
+            if not conf.get(key, None) or conf[key] == "==error==":
+                conf[key] = getattr(self, key)
+        conf["version"] = __version__
+
+        # Write config. Will save before custom setting.
         try:
             with open(
-                cls.Conf_Path, "w" if os.path.isfile(cls.Conf_Path) else "x"
+                self.config_path, "w" if os.path.isfile(self.config_path) else "x"
             ) as f:
-                f.write(cls.Config_Template.format(version=__version__, **vars(cls)))
+                f.write(self.CONFIG_TEMPLATE.format(**conf))
             print("Successful.")
         except Exception as e:
-            # print(str(e))
+            Log.error(str(e))
             print("Failed, create config.")
 
 
@@ -288,7 +323,7 @@ if CONFIG.warnings:
 #####################################################################
 # Implementation of additional functions.                           #
 #####################################################################
-def introduce():
+def introduce() -> None:
     """Print the description information."""
 
     # Print tools version and path.
@@ -304,6 +339,7 @@ def introduce():
     color_print("Path: ", Fx.b, end="")
     color_print("%s\n" % __file__, TermColor.SkyBlue, Fx.underline)
 
+    # Print description.
     color_print("Description:", Fx.b)
     color_print(
         (
@@ -311,7 +347,7 @@ def introduce():
             " Support Linux and MacOS. Partial support for windows.\n"
             "  It use short command to replace the original command, like: \n"
             "  `pigit ws` -> `git status --short`, `pigit b` -> `git branch`.\n"
-            "  Also you use `g -s` to get the all short command, have fun"
+            "  Also you use `pigit -s` to get the all short command, have fun"
             " and good lucky.\n"
             "  The open source path: %s" % (TermColor.SkyBlue + Fx.underline + __url__)
         ),
@@ -320,19 +356,12 @@ def introduce():
 
     print("\nYou can use ", end="")
     color_print("-h", TermColor.Green, end="")
-    print(" and ", end="")
+    print(" or ", end="")
     color_print("--help", TermColor.Green, end="")
     print(" to get help and more usage.\n")
 
 
-def init_hook():
-    try:
-        signal.signal(signal.SIGINT, leave)
-    except Exception as e:
-        print(str(e))
-
-
-def get_extra_cmds():
+def get_extra_cmds() -> dict:
     """Get custom cmds.
 
     Load the `extra_cmds.py` file under PIGIT HOME, check whether `extra_cmds`
@@ -343,9 +372,9 @@ def get_extra_cmds():
     """
     import imp
 
-    extra_cmd_path = PIGIT_HOME + "/extra_cmds.py"
-    # print(extra_cmd_path)
+    extra_cmd_path: str = PIGIT_HOME + "/extra_cmds.py"
     extra_cmds = {}
+
     if os.path.isfile(extra_cmd_path):
         try:
             extra_cmd = imp.load_source("extra_cmd", extra_cmd_path)
@@ -454,7 +483,7 @@ class CustomHelpFormatter(argparse.HelpFormatter):
 
 
 class Parser(object):
-    def __init__(self):
+    def __init__(self) -> None:
         super(Parser, self).__init__()
         self._parser = argparse.ArgumentParser(
             prog="pigit",
@@ -462,12 +491,9 @@ class Parser(object):
             prefix_chars="-",
             formatter_class=CustomHelpFormatter,
         )
-        self._parser.add_argument(
-            "-C",
-            "--complete",
-            action="store_true",
-            help="Add shell prompt script and exit.(Supported `bash`, `zsh`)",
-        )
+        self._add_arguments()
+
+    def _add_arguments(self) -> None:
         self._parser.add_argument(
             "-s",
             "--show-commands",
@@ -508,28 +534,38 @@ class Parser(object):
             const=".",
             type=str,
             metavar="PATH",
-            help=(
-                "Count the number of codes and output them in tabular form.\n"
-                "A given path can be accepted, and the default is the current directory."
-            ),
+            help="Count the number of codes and output them in tabular form."
+            "A given path can be accepted, and the default is the current directory.",
+        )
+        self._parser.add_argument(
+            "-C",
+            "--complete",
+            action="store_true",
+            help="Add shell prompt script and exit.(Supported `bash`, `zsh`, `fish`)",
         )
         self._parser.add_argument(
             "--create-ignore",
             type=str,
             metavar="TYPE",
             dest="ignore_type",
-            help="Create a demo .gitignore file. Need one argument, support: [%s]"
+            help="Create a demo `.gitignore` file. Need one argument, support: [%s]"
             % ", ".join(GitignoreGenetor.Supported_Types.keys()),
         )
         self._parser.add_argument(
             "--create-config",
             action="store_true",
-            help="Create a preconfigured file of git-tools.",
+            help="Create a preconfigured file of PIGIT."
+            "(If a profile exists, the values available in it are used)",
+        )
+        self._parser.add_argument(
+            "--shell",
+            action="store_true",
+            help="Go to the pigit shell mode.",
         )
         self._parser.add_argument(
             "--debug",
             action="store_true",
-            help="Run in debug mode.",
+            help="Current runtime in debug mode.",
         )
         self._parser.add_argument(
             "--out-log",
@@ -550,98 +586,152 @@ class Parser(object):
             "args", nargs="*", type=str, help="Command parameter list."
         )
 
-    def parse(self, custom_commands=None):
+    def parse(self, custom_commands: Optional[list] = None):
         if custom_commands:
-            return self._parser.parse_args(custom_commands)
-        return self._parser.parse_args()
+            return self._parser.parse_known_args(custom_commands)
+        return self._parser.parse_known_args()
+
+    def process(self, known_args, extra_unknown: Optional[list] = None):
+        try:
+            if known_args.complete:
+                completion_vars = {
+                    key: value.get("help", "") for key, value in Git_Cmds.items()
+                }
+                completion_vars.update(process_argparse(self._parser))
+                ShellCompletion(
+                    __project__, completion_vars, PIGIT_HOME
+                ).complete_and_use()
+                return
+
+            if known_args.config:
+                git_local_config()
+
+            if known_args.information:
+                repository_info(
+                    show_path=CONFIG.repository_show_path,
+                    show_remote=CONFIG.repository_show_remote,
+                    show_branches=CONFIG.repository_show_branchs,
+                    show_lastest_log=CONFIG.repository_show_lastest_log,
+                    show_summary=CONFIG.repository_show_summary,
+                )
+
+            if known_args.create_config:
+                CONFIG.create_config_template()
+                return
+
+            if known_args.ignore_type:
+                GitignoreGenetor(timeout=CONFIG.gitignore_generator_timeout,).launch(
+                    known_args.ignore_type,
+                    dir_path=Repository_Path,
+                )
+                return
+
+            if known_args.count:
+                path = known_args.count if known_args.count != "." else os.getcwd()
+                CodeCounter(
+                    count_path=path,
+                    use_ignore=CONFIG.codecounter_use_gitignore,
+                    result_saved_path=COUNTER_PATH,
+                    result_format=CONFIG.codecounter_result_format,
+                    use_icon=CONFIG.codecounter_show_icon,
+                ).count_and_format_print(
+                    show_invalid=CONFIG.codecounter_show_invalid,
+                )
+                return
+
+            git_processor = GitProcessor(
+                extra_cmds=get_extra_cmds(),
+                use_recommend=CONFIG.gitprocessor_use_recommend,
+                show_original=CONFIG.gitprocessor_show_original,
+                use_color=CONFIG.gitprocessor_interactive_color,
+                help_wait=CONFIG.gitprocessor_interactive_help_showtime,
+            )
+
+            def _shell_mode():
+                import pigit.tomato
+
+                print(
+                    "Welcome come PIGIT shell.\n"
+                    "You can use short commands directly. Input '?' to get help.\n"
+                )
+                while True:
+                    command = input("(pigit)> ").strip()
+                    if not command:
+                        continue
+
+                    command_args = command.split()
+                    head_command = command_args[0]
+
+                    if head_command in ["quit", "exit"]:  # ctrl+c
+                        break
+                    elif head_command in git_processor.cmds.keys():
+                        git_processor.process_command(head_command, command_args[1:])
+                    elif head_command == "tomato":
+                        # Tomato clock.
+                        pigit.tomato.main(command_args)
+                    elif head_command == "?":
+                        other_ = command_args[1:]
+                        if not other_:
+                            git_processor.command_help()
+                        else:
+                            print(other_)
+                            for item in other_:
+                                if item in git_processor.cmds.keys():
+                                    print(git_processor._generate_help_by_key(item))
+                                elif item == "tomato":
+                                    pigit.tomato.help("tomato")
+                    else:
+                        os.system(command)
+                return
+
+            if known_args.shell:
+                _shell_mode()
+
+            if known_args.show_commands:
+                return git_processor.command_help()
+
+            if known_args.command_type:
+                return git_processor.command_help_by_type(known_args.command_type)
+
+            if known_args.types:
+                return git_processor.type_help()
+
+            if known_args.command:
+                if known_args.command == "shell":
+                    return _shell_mode()
+                else:
+                    command = known_args.command
+                    known_args.args.extend(extra_unknown)  # type: list
+                    git_processor.process_command(command, known_args.args)
+                    return
+
+            # Don't have invalid command list.
+            if not list(filter(lambda x: x, vars(known_args).values())):
+                introduce()
+        except (KeyboardInterrupt, EOFError):
+            raise SystemExit(0)
+            pass
 
 
 @time_it
-def main(custom_commands=None):
-    init_hook()
+def main(custom_commands: Optional[list] = None):
+    # init_hook()
 
     parser = Parser()
     # parse custom comand, if has.
     if custom_commands is not None:
         stdargs = parser.parse(custom_commands)
-    stdargs = parser.parse()
-    # print(stdargs)
+    stdargs, extra_unknown = parser.parse()
+    # print(stdargs, extra_unknown)
 
     # Setup log handle.
     LogHandle.setup_logging(
-        debug=stdargs.debug,
+        debug=stdargs.debug or CONFIG.debug_mode,
         log_file=None if stdargs.out_log else LOG_PATH,
     )
 
-    if stdargs.complete:
-        completion_vars = {
-            key: value.get("help", "") for key, value in Git_Cmds.items()
-        }
-        completion_vars.update(process_argparse(parser._parser))
-        ShellCompletion(__project__, completion_vars, PIGIT_HOME).complete_and_use()
-        raise SystemExit(0)
-
-    if stdargs.config:
-        git_local_config()
-
-    if stdargs.information:
-        repository_info(
-            show_path=CONFIG.repository_show_path,
-            show_remote=CONFIG.repository_show_remote,
-            show_branches=CONFIG.repository_show_branchs,
-            show_lastest_log=CONFIG.repository_show_lastest_log,
-            show_summary=CONFIG.repository_show_summary,
-        )
-
-    if stdargs.create_config:
-        Config.create_config_template()
-        raise SystemExit(0)
-
-    if stdargs.ignore_type:
-        GitignoreGenetor(timeout=CONFIG.gitignore_generator_timeout,).create_gitignore(
-            stdargs.ignore_type,
-            dir_path=Repository_Path,
-        )
-        raise SystemExit(0)
-
-    if stdargs.count:
-        path = stdargs.count if stdargs.count != "." else os.getcwd()
-        CodeCounter(
-            count_path=path,
-            use_ignore=CONFIG.codecounter_use_gitignore,
-            result_saved_path=COUNTER_PATH,
-            result_format=CONFIG.codecounter_result_format,
-        ).count_and_format_print(
-            show_invalid=CONFIG.codecounter_show_invalid,
-        )
-        raise SystemExit(0)
-
-    git_processor = GitProcessor(
-        extra_cmds=get_extra_cmds(),
-        use_recommend=CONFIG.gitprocessor_use_recommend,
-        show_original=CONFIG.gitprocessor_show_original,
-        use_color=CONFIG.gitprocessor_interactive_color,
-        help_wait=CONFIG.gitprocessor_interactive_help_showtime,
-    )
-    if stdargs.show_commands:
-        git_processor.command_help()
-        raise SystemExit(0)
-
-    if stdargs.command_type:
-        git_processor.command_help_by_type(stdargs.command_type)
-        raise SystemExit(0)
-
-    if stdargs.types:
-        git_processor.type_help()
-        raise SystemExit(0)
-
-    if stdargs.command:
-        command = stdargs.command
-        git_processor.process_command(command, stdargs.args)
-        raise SystemExit(0)
-
-    if not list(filter(lambda x: x, vars(stdargs).values())):
-        introduce()
+    # process result.
+    parser.process(stdargs, extra_unknown)
 
 
 if __name__ == "__main__":
